//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _ADCC
#define _ADCC

// HEADERS ==========================================================
#include "adc.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void adc_init(void)
//----------------------------------------------------------------------------------
{
   output_float(I2C_SCL);
   output_float(I2C_SDA);
}

//----------------------------------------------------------------------------------
bool adc_write_bytes(int8 address, int8 function)
//----------------------------------------------------------------------------------
{
	bool ack = true;

	i2c_start();
	ack &= !i2c_write(address);
	ack &= !i2c_write(function);

	return ack;
}

//----------------------------------------------------------------------------------
bool adc_read_bytes(int8 address, unsigned int16 * value, bool dual)
//----------------------------------------------------------------------------------
{
	bool ack = true;
	unsigned int8 reading_high;
	unsigned int8 reading_low;

	i2c_start();
	ack &= !i2c_write(address + 1);

	//Don't acknowledge the last byte...
	reading_high = i2c_read(true);
	reading_low = i2c_read(dual);
	*value = make16(reading_high, reading_low);

	if (dual)
	{
		reading_high = i2c_read(true);
		reading_low = i2c_read(false);
		value++;
		*value = make16(reading_high, reading_low);
	}

	i2c_stop();

	return ack;
}

//----------------------------------------------------------------------------------
unsigned int16 convert_units(unsigned int16 adc)
//----------------------------------------------------------------------------------
{
	//now convert into correct units without rounding too much
	/*
	Pa			mbar		mmH2O		InH20
	1000		10			101.974		4.015
	1000000		10000		101974		4015
	*/

	/*
	//but we are using scaled integers rather than floating point so
	Pa (0)		mbar (2)	mmH2O (1)	InH20 (2)	//number in brackets is the number of decimals we are representing
	1000		1000		1019.74		401.5
	100000		100000		101974		40150
	so take result in Pa and multiply by say (101974) then divide by 100000
	but will the intermediate results fit in an int32?
	Largest Pascal that we could possibly measure is 2500
	so  2500 * 101974 =   254,935,000
	and 2^32          = 4,294,967,296
	so yes, it fits easily!!!
	*/

	unsigned int32 temp;

	temp = adc;

	switch (units)
	{
		case st_units_pa:
			//nothing to do
			break;
		case st_units_mbar:
			//nothing to do
			break;
		case st_units_mmh2o:
			temp *= 101974;
			temp /= 100000;
			break;
		case st_units_inh2o:
			temp *= 40150;
			temp /= 100000;
			break;
	}

	return temp;
}

//----------------------------------------------------------------------------------
unsigned int16 scale_pressure(unsigned int16 adc, unsigned int16 offset, unsigned int32 slope, bool pascal)
//----------------------------------------------------------------------------------
{
	unsigned int32 temp;

	temp = (int32)(adc & 0x0FFF); //Mask off the top nibble

	//now convert from adc counts into Pascal
	temp *= 2500;  //pressure sensor reads max output for 2500Pa pressure
	temp /= 4096;  //12 bit ADC

	//check that we are not likely to become negative
	if (temp < offset)
	{
		temp = offset;
	}
   
	//now apply offset and slope corrections
	temp -= offset;				//apply offset
	temp *= slope;				//apply slope
	temp /= 100000;				//adjust scale

	if (!pascal)
	{
		temp = convert_units((unsigned int16)temp);
	}

	return temp;
}

//----------------------------------------------------------------------------------
bool read_single_pressure(unsigned int16 * p1)
//----------------------------------------------------------------------------------
{
	unsigned int16 adc;
	bool ok;

	adc_write_bytes(ADC_ADDRESS, ADC_SINGLE);
	delay_us(5);
	adc_write_bytes(ADC_ADDRESS, 0);
	ok = adc_read_bytes(ADC_ADDRESS, &adc, false);
	pascal_pressure[0] = scale_pressure(adc, cal_offset[0], cal_slope[0], true);
	*p1 = scale_pressure(adc, cal_offset[0], cal_slope[0], false);

	return ok;
}

//----------------------------------------------------------------------------------
bool read_dual_pressures(unsigned int16 * p1, unsigned int16 * p2)
//----------------------------------------------------------------------------------
{
	unsigned int16 adc[2];
	bool ok;

	adc_write_bytes(ADC_ADDRESS, ADC_DUAL);
	ok = adc_read_bytes(ADC_ADDRESS, &adc[0], true);

	//Swapped over the readings for dual channel so the sensors are effectively swapped on the board
	pascal_pressure[0] = scale_pressure(adc[1], cal_offset[0], cal_slope[0], true);
	*p1 = scale_pressure(adc[1], cal_offset[0], cal_slope[0], false);
	pascal_pressure[1] = scale_pressure(adc[0], cal_offset[1], cal_slope[1], true);
	*p2 = scale_pressure(adc[0], cal_offset[1], cal_slope[1], false);

	return ok;
}

// =============================================================
#endif
